#!/bin/bash
set +ex

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start time :"

if [ -z "$1" ] || [ -z "$2" ]
then
    echo "You have to pass a device serial and packagearchive link"
    echo "$0 04bf2c50e294d599 http://host/path/to/archive.zip"
    exit -1
fi

exec_with_adb() {
    adb -s $ANDROID_SERIAL shell "$@"
}

exec_with_adb_user() {
    adb -s $ANDROID_SERIAL shell sudo -i -u phablet bash -ic "$@"
}

ANDROID_SERIAL=$1
packagearchive=$2
zip_name=`basename $packagearchive`
mir_rc=0
reboot_wait=30
network_wait=60
max_retries=5
results_dir=./results
mir_dir=$results_dir/mir
crash_dir=$results_dir/crash
log_dir=$results_dir/log
benchmark_dir=$results_dir/benchmark_results
benchmark_results=/tmp/glmark2_fullscreen_default.results

HOME_DIR="/home/phablet"
setup_dir="$HOME_DIR/mir-testrunner"

### Device initiation ###
# First reboot
echo "Rebooting the phone will take approximately $reboot_wait seconds to settle"
adb -s $ANDROID_SERIAL reboot
adb -s $ANDROID_SERIAL wait-for-device
sleep $reboot_wait

# Set developer mode
rm .dev_mode || true
adb -s $ANDROID_SERIAL pull /.dev_mode
if [[ ! -a .dev_mode ]]; then
    exec_with_adb "touch /userdata/.writable_image"
    echo "Rebooting the phone to set dev mode."
    echo "It will take approximately $reboot_wait seconds to settle"
    adb -s $ANDROID_SERIAL reboot
    adb -s $ANDROID_SERIAL wait-for-device
    sleep $reboot_wait
    exec_with_adb "touch /.dev_mode"
fi

# Disable the intro screen
rm .intro_off || true
adb -s $ANDROID_SERIAL pull /.intro_off
if [[ ! -a .intro_off ]]; then
    echo "Disable welcome and edges intro"
    exec_with_adb_user "mkdir -p ~/.config/ubuntu-system-settings"
    exec_with_adb_user "touch ~/.config/ubuntu-system-settings/wizard-has-run"
    exec_with_adb "dbus-send --system --print-reply --dest=org.freedesktop.Accounts /org/freedesktop/Accounts/User32011 org.freedesktop.DBus.Properties.Set string:com.canonical.unity.AccountsService string:demo-edges variant:boolean:false"

    exec_with_adb "touch /.intro_off"
    exec_with_adb "cat /var/lib/AccountsService/users/phablet"
    echo "Rebooting the phone to disable intro mode."
    echo "It will take approximately $reboot_wait seconds to settle"

    adb -s $ANDROID_SERIAL reboot
    adb -s $ANDROID_SERIAL wait-for-device
    sleep $reboot_wait
fi

### Cleanup ###
# Cleanup host
rm $zip_name
rm -rf $results_dir
rm -rf package_archive || true
mkdir -p $results_dir
mkdir -p $benchmark_dir
mkdir -p $mir_dir
mkdir -p $crash_dir
mkdir -p $log_dir

# Cleanup target
exec_with_adb "rm -r $setup_dir $HOME_DIR/output"
exec_with_adb "rm /var/crash/*"

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start Setup target :"

### Setup target ###
# Setup scripts
exec_with_adb_user "mkdir -p $setup_dir"
adb -s $ANDROID_SERIAL push mir_install_packages.sh "$setup_dir"

# Setup network
echo "Pausing $network_wait seconds before starting network"
sleep $network_wait
exec_with_adb 'nmcli dev wifi connect ubuntu-qa-g-wpa-d password qalabwireless'

# Collect pre-setup info
adb -s $ANDROID_SERIAL pull /etc/media-info $log_dir

# Setup package archive
wget $packagearchive
unzip $zip_name -d package_archive
if [ -d package_archive/output ]; then
    ARCHIVE_DIR=package_archive/output
    chmod 755 $ARCHIVE_DIR
elif [ -d package_archive/archive/work/output ]; then
    ARCHIVE_DIR=package_archive/archive/work/output
    chmod 755 $ARCHIVE_DIR
else
    echo "E: Cannot find local package archive" >&2
    exit 1
fi
adb -s $ANDROID_SERIAL push $ARCHIVE_DIR $HOME_DIR/archive

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start PPAs and local Archive :"

# Setup the PPAs and local Archive
ppa_repos="ppa_sources"
rm $ppa_repos || true
touch $ppa_repos
for h in $hooks; do
    echo "Processing $h"
    if echo $h | grep D09add_ppa; then
        ppa=`echo $h | sed 's/D09add_ppa~\(.*\)~\(.*\)/ppa:\1\/\2/'`
	echo ${ppa} >> ${ppa_repos}
    fi
done

local_archive="deb $local_archive_source/$local_archive_name $local_archive_pocket/ "
if echo $local_archive | grep "http"; then
    local_archive=`sh -c "echo $local_archive | sed \"s/naartjie/10.98.3.7/\""`
    echo ${local_archive} >> ${ppa_repos}
fi
echo "Host side PPA repo list:"
cat $ppa_repos
adb -s $ANDROID_SERIAL push ${ppa_repos} /tmp/${ppa_repos}

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start Add the test packages :"

# Add the test packages (these are the extra packages specified in the job cfg)
exec_with_adb "sudo apt-get -y --force-yes --no-install-recommends install $test_packages"


exec_with_adb "$setup_dir/mir_install_packages.sh"

### Pre-run ###
# Collect pre-run logs
exec_with_adb "top -n 1 -b | head -20 > /tmp/top-pre.log"
adb -s $ANDROID_SERIAL pull /tmp/top-pre.log $log_dir
exec_with_adb "dpkg -l > /tmp/dpkg-l.log"
adb -s $ANDROID_SERIAL pull /tmp/dpkg-l.log $log_dir

date --utc --iso-8601=seconds | xargs echo "[timestamp] Start Run Tests :"

### Run Tests ###
# Setup to run the tests
rm .setup_complete || true
adb -s $ANDROID_SERIAL pull /.setup_complete
if [[ ! -a .setup_complete ]]; then
    echo "SETUP FAILED - CAN'T CONTINUE"
    mir_rc=1
else
    # Connect to the unrestricted network for running tests
    exec_with_adb 'nmcli dev wifi connect ubuntu-qa-g-wpa-d password qalabwireless'

    # Execute all of the test suites
    echo "I: Ready to run tests"
    echo "Rebooting the phone will take approximately $reboot_wait seconds to settle"
    adb -s $ANDROID_SERIAL reboot
    adb -s $ANDROID_SERIAL wait-for-device
    phablet-network --skip-setup -t 90s
    sleep $reboot_wait
    echo "I: Turning on the display"
    exec_with_adb "powerd-cli display on bright &"

    echo "I: stopping unity8"
    exec_with_adb_user "/sbin/initctl stop unity8"
    echo "I: stopping lightdm"
    exec_with_adb "/sbin/initctl stop lightdm"
    
    adb -s $ANDROID_SERIAL wait-for-device

    set -x 
    for suite in $test_suites; do
        echo running suite $suite
        date --utc --iso-8601=seconds | xargs echo "[timestamp] Start :" $suite
        if grep -q mir_demo_client_ <<< $suite  ; then
            if ! phablet-test-run -x -s $ANDROID_SERIAL $extra -v  mir-demo-tester --mir-demo $suite /tmp/$suite-log ; then
                mir_rc=-1
            fi

        else
            if ! phablet-test-run -x -s $ANDROID_SERIAL $extra -v  $suite ; then
                mir_rc=-1
            fi
        fi

        date --utc --iso-8601=seconds | xargs echo "[timestamp] End :" $suite
    done

    adb -s $ANDROID_SERIAL wait-for-device
fi

### Post Run ###
# Collect post-run logs
exec_with_adb "top -n 1 -b | head -20 > /tmp/top-post.log"
adb -s $ANDROID_SERIAL pull /tmp/top-post.log $log_dir
exec_with_adb "dmesg > /tmp/dmesg.log"
adb -s $ANDROID_SERIAL pull /tmp/dmesg.log $log_dir

for suite in $test_suites; do
    adb pull /tmp/$suite-log $mir_dir || true
done

# Collect crash files
echo "I: Gathering crash files (if any)"
adb -s $ANDROID_SERIAL pull /var/crash $crash_dir
ls $crash_dir

# Collect benchmark results
adb -s $ANDROID_SERIAL pull $benchmark_results $benchmark_dir

date --utc --iso-8601=seconds | xargs echo "[timestamp] End time :"

echo "I: Testing complete with returncode $mir_rc"
exit $mir_rc
